
// This file is originally from Trantor - WindowsSupport.cc

// Copyright (c) 2016-2021, Tao An.  All rights reserved.

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
//
//   * Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
//   * Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//   * Neither the name of Tao An nor the names of other contributors
// may be used to endorse or promote products derived from this software
// without specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
// "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
// LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
// A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
// OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
// SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
// LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
// DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
// THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
// OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

#include "windows_support.h"
#include <winsock2.h>

// from polipo
int win32_read_socket(int fd, void *buf, int n) {
	int rc = recv(fd, reinterpret_cast<char *>(buf), n, 0);
	if (rc == SOCKET_ERROR) {
		_set_errno(WSAGetLastError());
	}
	return rc;
}

int readv(int fd, const struct iovec *vector, int count) {
	int ret = 0; /* Return value */
	int i;
	for (i = 0; i < count; i++) {
		int n = vector[i].iov_len;
		int rc = win32_read_socket(fd, vector[i].iov_base, n);
		if (rc == n) {
			ret += rc;
		} else {
			if (rc < 0) {
				ret = (ret == 0 ? rc : ret);
			} else {
				ret += rc;
			}
			break;
		}
	}
	return ret;
}
